local HouseAddition = ...

--[[
-- Build the assignable function asset for the manual placed buildings
function registerAssignableFunction(_assetId)
    -- mandatory functions
    HouseAddition:registerAsset({
		DataType = "BUILDING_FUNCTION_LIST",
                Id = _HomePrefix .. "_" .. _assetId .. "_FUNCTION_LIST",
                AssetFunctionList = {
                        registerFunctionHouse(_HomePrefix, _assetId, "1"),
                        registerFunctionHouse(_HomePrefix, _assetId, "2")
                }
    })
    -- optional functions if the mod "Cottage" is present
    if (_flagCottage) then
        HouseAddition:overrideAsset({
		DataType = "BUILDING_FUNCTION_LIST",
                Id = _HomePrefix .. "_" .. _assetId .. "_FUNCTION_LIST",
                AssetFunctionList = {
                    Action = "APPEND",
                    "BUILDING_FUNCTION_COTTAGE_BERRIES",
                    "BUILDING_FUNCTION_COTTAGE_WOOD",
                    "BUILDING_FUNCTION_FORESTER",
                    "BUILDING_FUNCTION_COTTAGE_FRUIT",
                    "BUILDING_FUNCTION_COTTAGE_BREAD",
                    "BUILDING_FUNCTION_COTTAGE_CHEESE",
                    "BUILDING_FUNCTION_COTTAGE_CLOTH",
                    "BUILDING_FUNCTION_COTTAGE_CLOTHES",
                    "BUILDING_FUNCTION_COTTAGE_FURNITURE",
                    "BUILDING_FUNCTION_HUNTERS_HUT",
                }
        })
        -- extra optional functions if the mod "Vegetable Farms" is present
        if (_flagVegetableFarms) then
            HouseAddition:overrideAsset({
            Id = _HomePrefix .. "_" .. _assetId .. "_FUNCTION_LIST",
                AssetFunctionList = {
                    Action = "APPEND",
                    "BUILDING_FUNCTION_COTTAGE_BEAN",
                    "BUILDING_FUNCTION_COTTAGE_CABBAGE",
                }
            })
        end
    end
    -- register the assignable function asset
    HouseAddition:registerAsset({
		DataType = "BUILDING_FUNCTION_ASSIGNABLE",
                Id = _HomePrefix .. "_" .. _assetId .. "_ASSIGNABLE_FUNCTION",
                AuthorizedFunctionList = _HomePrefix .. "_" .. _assetId .. "_FUNCTION_LIST"
    })
    return _HomePrefix .. "_" .. _assetId .. "_ASSIGNABLE_FUNCTION"
end
--]]

-- register the simple building parts
function registerHouseBuilding(_fbxName, _buildingAssetId, _buildingZone, _villagerCapacity)
-- Apply building asset processor
    HouseAddition:registerAssetProcessor(_modelsFolder .. "/" .. _fbxName .. ".fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
    })

-- Register default roof material
    HouseAddition:registerAssetId(_modelsFolder .. "/" .. _fbxName .. ".fbx/Materials/roof", _HomePrefix .. "_" .. _buildingAssetId .. "_DEFAULT_MATERIAL")

-- Register common prefab nodes
    HouseAddition:registerAssetId(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/homePart", _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB")

-- Register buildingPart for random placement
    HouseAddition:registerAsset({
	DataType = "BUILDING_PART",
	Id = _HomePrefix .. "_" .. _buildingAssetId .. "_PART",
	Name = _HomePrefix .. "_" .. _buildingAssetId .. "_NAME",
	Description = _HomePrefix .. "_" .. _buildingAssetId .. "_DESC",
	Category = "CORE",
	BuildingZone = _buildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB"
	},
	Cost = { -- added to basic costs of the housing system
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{
			{ Resource = "WOOD", Quantity = 0 }
		}}
	}
    })
--[[
    -- if the mod "Cottage" is present, adds its functions 
    if _flagCottage then
        HouseAddition:overrideAsset({
	    DataType = "BUILDING_PART",
	    Id = _HomePrefix .. "_" .. _buildingAssetId .. "_PART",
	    AssetBuildingFunction = FUNCTION_LIST_COTTAGE
        })
    end
--]]
-- Register buildingPart for manual placement
    function registerHouseLvlFunction(_lvl)
        HouseAddition:registerAsset({
	      DataType = "BUILDING_PART",
	      Id = _HomePrefix .. "_PLACEABLE_LVL" .. _lvl .. "_" .. _buildingAssetId .. "_PART",
	      Name = _HomePrefix .. "_"  .. _buildingAssetId .. "_NAME",
	      Description = _HomePrefix .. "_"  .. _buildingAssetId .. "_DESC",
	      Category = "CORE",
	      BuildingZone = _buildingZone,
	      ConstructorData = {
		    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		    CoreObjectPrefab = _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB"
	      },
	      Cost = {
		    UpkeepCost = {
		        { Resource = "GOLD", Quantity = _NoSandbox*0 }
		    },
		    ResourceNeededList = {{
			  { Resource = "WOOD", Quantity = _NoSandbox*10 }
		    }}
	      },
            VillagerCapacity = _villagerCapacity,
      	AssetBuildingFunction = registerFunctionHouse(_HomePrefix, _buildingAssetId, _lvl),
        })
    end
    registerHouseLvlFunction("1")
    registerHouseLvlFunction("2")

-- Building grounded
    HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/homePart", {
	DataType = "COMP_GROUNDED"
    })
    HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/homePart/step_0/porte", {
	DataType = "COMP_GROUNDED"
    })

-- The fireplace...
    HouseAddition:registerAssetId(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/homePart/step_2/cheminee/SmokeEmitter", _HomePrefix .. "_" .._buildingAssetId .. "_SMOKE_EMITTER_PREFAB")

    HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/homePart", {
	DataType = "COMP_BUILDING_PART",
	FeedbackComponentListToActivate = {
                 { _HomePrefix .. "_" .. _buildingAssetId .. "_SMOKE_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" }
	}
    })

    HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/homePart/step_2/cheminee/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "PARTICLE_SYSTEM_HOUSING_SMOKE",
	IsPlaying = true,
	IsEmitting = true
    })

    HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/homePart/step_2/cheminee/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
	Enabled = false
    })
end

registerHouseBuilding("Home2", "HOME2",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {3, 7} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 4)
registerHouseBuilding("Home3", "HOME3",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 8}, {2, 0} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } },
                          { Polygon = polygon.createRectangle( {8, 4}, {0, 2} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 6)
registerHouseBuilding("Home4", "HOME4",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 8} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 6)
registerHouseBuilding("Home5", "HOME5",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 6} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 6)
registerHouseBuilding("Home6", "HOME6",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 7} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 6)

registerHouseBuilding("Home7", "HOME7",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 6} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 8)

--------------------------------------------------------------------------
-- more complex building with door and windows randomly placed 
function registerBuildingAssemblage(_fbxName, _buildingAssetId, _buildingZone, _villagerCapacity)
-- Apply building asset processor
    HouseAddition:registerAssetProcessor(_modelsFolder .. "/" .. _fbxName .. ".fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
    })

-- Register default roof material
    HouseAddition:registerAssetId(_modelsFolder .. "/" .. _fbxName .. ".fbx/Materials/roof", _HomePrefix .. "_" .. _buildingAssetId .. "_DEFAULT_MATERIAL")
 
-- Register prefab nodes
    HouseAddition:registerAssetId(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart", _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB")

-- Register the assemblage part for random part
        HouseAddition:registerAsset({
	      DataType = "BUILDING_PART",
	      Id = _HomePrefix .. "_" .. _buildingAssetId .. "_PART",
	      Name = _HomePrefix .. "_" .. _buildingAssetId .. "_NAME",
	      Description = _HomePrefix .. "_" .. _buildingAssetId .. "_DESC",
	      Category = "CORE",
	      BuildingZone = _buildingZone,
	      ConstructorData = {
		    DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
		    CoreRandomBuildingPartList = { _HomePrefix .. "_" .. _buildingAssetId .. "_PREFAB" },
                MandatoryBuildingPartList = {
                    { BuildingPart = _HomePrefix .. "_WOODEN_DOOR_PART" },
                    { BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 },
                    { BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 },
                    { BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 },
                    { BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 },
                    { BuildingPart = _HomePrefix .. "_FENETRE_PART", Probability = 0.5 },
                },
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	      },
	      Cost = {
		    UpkeepCost = {
			  { Resource = "GOLD", Quantity = 0 }
		    },
		    ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			  { Resource = "WOOD", Quantity = 0 }
		    }}
	      }
        })

    function registerHouseLvlFunction(_lvl)
-- Register the assemblage part for manual part
        HouseAddition:registerAsset({
	      DataType = "BUILDING_PART",
	      Id = _HomePrefix .. "_PLACEABLE_LVL" .. _lvl .. "_" .. _buildingAssetId .. "_PART",
	      Name = _HomePrefix .. "_" .. _buildingAssetId .. "_NAME",
	      Description = _HomePrefix .. "_" .. _buildingAssetId .. "_DESC",
	      Category = "CORE",
	      BuildingZone = _buildingZone,
	      ConstructorData = {
		    DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
		    RandomPartList = { _HomePrefix .. "_" .. _buildingAssetId .. "_PART" },
	      },
            Cost = {
		    UpkeepCost = {
			  { Resource = "GOLD", Quantity = _NoSandbox*0 }
		    },
		    ResourceNeededList = {{
			  { Resource = "WOOD", Quantity = _NoSandbox*10 }
		    }}
	      },
            VillagerCapacity = _villagerCapacity,
	      AssetBuildingFunction = registerFunctionHouse(_HomePrefix, _buildingAssetId, _lvl),
        })
    end
    registerHouseLvlFunction("1")
    registerHouseLvlFunction("2")

    HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart", {
	DataType = "COMP_GROUNDED"
    })

-- several models of door
    HouseAddition:registerAsset({
	DataType = "BUILDING_PART",
	Id = _HomePrefix .. "_WOODEN_DOOR_PART",
	ConstructorData = {
		          DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
		          RandomPartList = {
                                _HomePrefix .. "_DOOR1_PART",
                                _HomePrefix .. "_DOOR2_PART",
                                _HomePrefix .. "_DOOR3_PART",
                          }
        }
    })

-- Register the door parts
    function registerPortePart(_doorName, _doorAssetId, _doorZone)
        HouseAddition:registerAssetId(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/" .. _doorName .. "Part", _HomePrefix .. _doorAssetId .. "_PREFAB")

        HouseAddition:registerAsset({
        	DataType = "BUILDING_PART",
	        Id = _HomePrefix .. _doorAssetId .. "_PART",
	        ConstructorData = {
		    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		    CoreObjectPrefab = _HomePrefix .. _doorAssetId .. "_PREFAB"
	        }
        })

        HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/" .. _doorName .. "Part", {
	    DataType = "COMP_BUILDING_PART",
            BuildingPartType = "MAJOR"
        })

        HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/" .. _doorName .. "Part", {
	    DataType = "COMP_GROUNDED"
        })
    end

    registerPortePart("door1", "_DOOR1", { 0.1, 1 })
    registerPortePart("door2", "_DOOR2", { 0.1, 1 })
    registerPortePart("door3", "_DOOR3", { 1, 1 })

-- Register the window part
    HouseAddition:registerAssetId(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/fenetrePart", _HomePrefix .. "_FENETRE_PREFAB")

    HouseAddition:registerAsset({
	DataType = "BUILDING_PART",
	Id = _HomePrefix .. "_FENETRE_PART",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = _HomePrefix .. "_FENETRE_PREFAB"
	}
    })

    HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/fenetrePart", {
	DataType = "COMP_BUILDING_PART",
        BuildingPartType = "MAJOR",
    })

-- The fireplace...
    HouseAddition:registerAssetId(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart/step_2/cheminee/SmokeEmitter", _HomePrefix .. "_" .. _buildingAssetId .. "_SMOKE_EMITTER_PREFAB")

    HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart", {
	DataType = "COMP_BUILDING_PART",
	FeedbackComponentListToActivate = {
                 { _HomePrefix .. "_" .. _buildingAssetId .. "_SMOKE_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" }
	}
    })

    HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart/step_2/cheminee/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "PARTICLE_SYSTEM_HOUSING_SMOKE",
	IsPlaying = true,
	IsEmitting = true
    })

    HouseAddition:registerPrefabComponent(_modelsFolder .. "/" .. _fbxName .. ".fbx/Prefab/assemblyHouse/homePart/step_2/cheminee/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
	Enabled = false
    })
end

registerBuildingAssemblage("Home11", "HOME11",
                      { ZoneEntryList = {
                          { Polygon = polygon.createRectangle( {4, 8} ),
                            Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
                        }
                      }, 6)
