local HouseAddition = ...

-- Register the building part
HouseAddition:dofile(_scriptFolder .. "/newGate.lua", _HomePrefix)

-- Register the monument for manual setup
function registerSubBuilding(_suffix, _assetBuildingPartList)
    HouseAddition:registerAsset({
		DataType = "BUILDING",
		Id = _HomePrefix .. "_" .. _suffix,
		Name = _HomePrefix .. "_" .. _suffix .. "_NAME",
		Description = _HomePrefix .. "_" .. _suffix .. "_DESC",
		BuildingType = BUILDING_TYPE.MONUMENT,
		AssetCoreBuildingPart = "BUILDING_PART_MONUMENT_POLE",
		AssetBuildingPartList = _assetBuildingPartList,
		RequiredPartList = {
			-- Necessarily a list of 'MONUMENT_REQUIRED_PART_PAIR', no need to specify the DataType of each element of the list
			{ Category = "CORE", Min = 1 },
		},
		IsManuallyUnlocked = false,
		IsDestructible = true,
		IsEditable = false,
		IsClearTrees = true,
    })
    HouseAddition:overrideAsset({
        Id = "TAXATION_FUNCTION_HOUSING",
        AssetBuildingList = {
            Action = "APPEND",
            _HomePrefix .. "_" .. _suffix,
        },
    })
end
registerSubBuilding("HOUSE_LVL1",
                    {
		            _HomePrefix .. "_PLACEABLE_LVL1_HOME2_PART",
		            _HomePrefix .. "_PLACEABLE_LVL1_HOME3_PART",
		            _HomePrefix .. "_PLACEABLE_LVL1_HOME4_PART",
		            _HomePrefix .. "_PLACEABLE_LVL1_HOME5_PART",
		            _HomePrefix .. "_PLACEABLE_LVL1_HOME6_PART",
		            _HomePrefix .. "_PLACEABLE_LVL1_HOME11_PART",
		            _HomePrefix .. "_PLACEABLE_LVL1_HOME7_PART",
                        _HomePrefix .. "_LVL1_NEW_GATE_PART",
                    }
)
registerSubBuilding("HOUSE_LVL2",
                     {
		            _HomePrefix .. "_PLACEABLE_LVL2_HOME2_PART",
		            _HomePrefix .. "_PLACEABLE_LVL2_HOME3_PART",
		            _HomePrefix .. "_PLACEABLE_LVL2_HOME4_PART",
		            _HomePrefix .. "_PLACEABLE_LVL2_HOME5_PART",
		            _HomePrefix .. "_PLACEABLE_LVL2_HOME6_PART",
		            _HomePrefix .. "_PLACEABLE_LVL2_HOME11_PART",
		            _HomePrefix .. "_PLACEABLE_LVL2_HOME7_PART",
                        _HomePrefix .. "_LVL2_NEW_GATE_PART",
                    }
)

function registerSubBuildingVanilla(_assetPrefix, _suffix, _lvl)
    function registerFunctionHouse()
        function houseStatusList()
            function houseStatus(_desirabilityLevel)
                return {
	            DataType = "HOUSE_STATUS",
	            RequiredDesirability = _desirabilityLevel,
	            ResourceGenerated = "HOUSING_LEVEL_" .. _lvl,
	            HouseDensityPartList = { _suffix }
                }
            end
            if _lvl == "1" then
                return {
	            houseStatus(DESIRABILITY_LEVEL.LOW),
	            houseStatus(DESIRABILITY_LEVEL.HIGH)
	        }
            end
            if _lvl == "2" then
	        return {
	            houseStatus(DESIRABILITY_LEVEL.AVERAGE),
	            houseStatus(DESIRABILITY_LEVEL.VERY_HIGH)
	        }
            end
            return nil
        end

        HouseAddition:registerAsset({
	    DataType= "HOUSE_SETUP",
	    Id = _assetPrefix .. "_" .. _suffix .. "_HOUSING_LVL" .. _lvl .. "_SETUP",
	    HouseStatusList = houseStatusList()
        })
        HouseAddition:registerAsset({
	    DataType = "BUILDING_FUNCTION_HOUSE",
	    Id = _assetPrefix .. "_" .. _suffix .. "_HOUSING_LVL" .. _lvl .. "_FUNCTION",
            Name = _assetPrefix .. "_BUILDING_FUNCTION_HOUSING_LVL" .. _lvl .. "_NAME",
	    RelatedZone = "RESIDENTIAL",
	    RelatedDesirability = "DESIRABILITY_RESIDENTIAL",
	    AssetHouseSetup =  _assetPrefix .. "_" .. _suffix .. "_HOUSING_LVL" .. _lvl .. "_SETUP",
        })
        return _assetPrefix .. "_" .. _suffix .. "_HOUSING_LVL" .. _lvl .. "_FUNCTION"
    end
    HouseAddition:registerAsset({
		DataType = "BUILDING",
		Id = _assetPrefix .. "_" .. _suffix .. "_" .. _lvl,
		Name = _assetPrefix .. "_" .. _suffix .. "_" .. _lvl .. "_NAME",
		Description = _assetPrefix .. "_" .. _suffix .. "_" .. _lvl .. "_DESC",
		BuildingType = BUILDING_TYPE.MONUMENT,
		AssetCoreBuildingPart = "BUILDING_PART_MONUMENT_POLE",
		AssetBuildingPartList = { _suffix, }, 
--            VillagerCapacity = 6,
      	AssetBuildingFunction = registerFunctionHouse(),
		IsManuallyUnlocked = false,
		IsDestructible = true,
		IsEditable = false,
		IsClearTrees = true,
    })
    HouseAddition:overrideAsset({
        Id = "TAXATION_FUNCTION_HOUSING",
        AssetBuildingList = {
            Action = "APPEND",
            _assetPrefix .. "_" .. _suffix .. "_" .. _lvl,
        },
    })
end
registerSubBuildingVanilla(_HomePrefix, "BUILDING_PART_HOUSING_SERF_DENSITY_1", "1")
registerSubBuildingVanilla(_HomePrefix, "BUILDING_PART_HOUSING_SERF_DENSITY_2", "1")
registerSubBuildingVanilla(_HomePrefix, "BUILDING_PART_HOUSING_CITIZEN_DENSITY_1", "2")
registerSubBuildingVanilla(_HomePrefix, "BUILDING_PART_HOUSING_CITIZEN_DENSITY_2", "2")

-- Register optional materials
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/redRoof", _HomePrefix .. "_DECORATION_RED_TILE_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/blueRoof", _HomePrefix .. "_DECORATION_BLUE_TILE_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/brownRoof", _HomePrefix .. "_DECORATION_BROWN_TILE_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/greyRoof", _HomePrefix .. "_DECORATION_LIGHT_GREY_TILE_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/grey2Roof", _HomePrefix .. "_DECORATION_DARK_GREY_TILE_ROOF_MATERIAL")

HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/wRedRoof", _HomePrefix .. "_DECORATION_RED_SHINGLE_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/wBlueRoof", _HomePrefix .. "_DECORATION_BLUE_SHINGLE_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/wBrownRoof", _HomePrefix .. "_DECORATION_BROWN_SHINGLE_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/wYellowRoof", _HomePrefix .. "_DECORATION_YELLOW_SHINGLE_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/wGreenRoof", _HomePrefix .. "_DECORATION_GREEN_SHINGLE_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/wManorRoof", _HomePrefix .. "_DECORATION_WOODEN_MANOR_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/wManor2Roof", _HomePrefix .. "_DECORATION_STONE_MANOR_ROOF_MATERIAL")

HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/brownMonasteryRoof", _HomePrefix .. "_DECORATION_BROWN_MONASTERY_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/blueMonasteryRoof", _HomePrefix .. "_DECORATION_BLUE_MONASTERY_ROOF_MATERIAL")
HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/purpleMonasteryRoof", _HomePrefix .. "_DECORATION_PURPLE_MONASTERY_ROOF_MATERIAL")

HouseAddition:registerAssetId(_modelsFolder .. "/utility/roofMaterials.fbx/Materials/strawRoof", _HomePrefix .. "_DECORATION_STRAW_ROOF_MATERIAL")

function configureMaterielSetList(_colorRoof, _isDefaultMaterial)
    function configureMaterial(_core, _colorMaterial)
        if _isDefaultMaterial then
            return  _HomePrefix .. "_" .. _core .. "_DEFAULT_MATERIAL"
        else
            return  _HomePrefix .. "_DECORATION_" .. _colorMaterial .. "_MATERIAL"
        end
    end
    return {
                SetName = _HomePrefix .. "_" .. _colorRoof,
                MaterialList = {
                    configureMaterial("HOME2", _colorRoof),
                    configureMaterial("HOME3", _colorRoof),
                    configureMaterial("HOME4", _colorRoof),
                    configureMaterial("HOME5", _colorRoof),
                    configureMaterial("HOME6", _colorRoof),
                    configureMaterial("HOME11", _colorRoof),
                    configureMaterial("HOME7", _colorRoof),
                    configureMaterial("NEW_GATE", _colorRoof),
                }
    }
end


HouseAddition:registerAsset({
    Id = _HomePrefix .. "_MONUMENT",
    DataType = "BUILDING",
    Name = _HomePrefix .. "_MONUMENT_NAME",
    Description = _HomePrefix .. "_MONUMENT_DESC",
    BuildingType = "MODS",
    AssetMiniatureBuildingPart = _HomePrefix .. "_NEW_GATE_PART",
    AssetCoreBuildingPart = "BUILDING_PART_MONUMENT_POLE",
    SubAssetBuildingList = {
		_HomePrefix .. "_HOUSE_LVL1",
		_HomePrefix .. "_HOUSE_LVL2",
            _HomePrefix .. "_BUILDING_PART_HOUSING_SERF_DENSITY_1_1",
            _HomePrefix .. "_BUILDING_PART_HOUSING_SERF_DENSITY_2_1",
            _HomePrefix .. "_BUILDING_PART_HOUSING_CITIZEN_DENSITY_1_2",
            _HomePrefix .. "_BUILDING_PART_HOUSING_CITIZEN_DENSITY_2_2",
    },
    AssetBuildingPartList = {},
    MaterialSetList = {
            configureMaterielSetList("DEFAULT_ROOF", true), -- default material list
            configureMaterielSetList("RED_TILE_ROOF", false),
            configureMaterielSetList("BLUE_TILE_ROOF", false),
            configureMaterielSetList("BROWN_TILE_ROOF", false),
            configureMaterielSetList("LIGHT_GREY_TILE_ROOF", false),
            configureMaterielSetList("DARK_GREY_TILE_ROOF", false),
            configureMaterielSetList("RED_SHINGLE_ROOF", false),
            configureMaterielSetList("BLUE_SHINGLE_ROOF", false),
            configureMaterielSetList("BROWN_SHINGLE_ROOF", false),
            configureMaterielSetList("YELLOW_SHINGLE_ROOF", false),
            configureMaterielSetList("GREEN_SHINGLE_ROOF", false),
            configureMaterielSetList("WOODEN_MANOR_ROOF", false),
            configureMaterielSetList("STONE_MANOR_ROOF", false),
            configureMaterielSetList("BROWN_MONASTERY_ROOF", false),
            configureMaterielSetList("BLUE_MONASTERY_ROOF", false),
            configureMaterielSetList("PURPLE_MONASTERY_ROOF", false),
            configureMaterielSetList("STRAW_ROOF", false),
    },
})

HouseAddition:registerAsset({
    DataType = "EVENT",
    Id = _HomePrefix .. "_EVENT_START",
    DaysToFirst = 1,
    ConditionList = {
        {
            DataType = "GAME_CONDITION_BUILDING_BUILT",
            AssetBuilding = "BUILDING_VILLAGE_CENTER",
        },
    },
    ActionList = {
        {
            DataType = "GAME_ACTION_UNLOCK_BUILDING_LIST",
            BuildingProgressData = {
                AssetBuildingList = {
                    _HomePrefix .. "_MONUMENT",
                },
            },
        }
    }
})

HouseAddition:overrideAsset({
    Id = "DEFAULT_BALANCING",
    EventList = {
        Action = "APPEND",
        _HomePrefix .. "_EVENT_START",
    }
})